import argparse
import os
import subprocess
import sys

def install_toolchain_and_sdk(version, install_dir):
    print(f"Installing SDK version {version} using sdk-manager...")
    result = subprocess.run([
        "nrfutil", "sdk-manager", "install", version, "--install-dir", install_dir
    ], capture_output=True, text=True)
    if result.returncode != 0:
        print("Error installing SDK:", result.stderr)
        sys.exit(1)
    print("SDK installed successfully.")

def launch_build(sdk_version, project_path, build_dir, board, config, install_dir, custom_board_dir):
    print(f"Setting up environment for SDK version {sdk_version}...")
    nrf_command = [
        "nrfutil", "sdk-manager", "toolchain", "launch",
        "--ncs-version", sdk_version,
        "--chdir", project_path,
        "--install-dir", install_dir,
        "--"
    ]
    west_command = [
        "west", "build",
        "--sysbuild",
        "--pristine",
        "-b", board,
        "-d", build_dir,
        project_path,
    ]
    cmake_args = []
    if config:
        cmake_args.append(f"-DCONF_FILE={config}")
    if custom_board_dir:
        cmake_args.append(f"-DBOARD_ROOT={custom_board_dir}")
    if cmake_args:
        west_command += ["--"] + cmake_args

    env = os.environ.copy()
    env["ZEPHYR_BASE"] = install_dir + '\\' + sdk_version + '\\' + "zephyr"
    print(env["ZEPHYR_BASE"])

    result = subprocess.run(nrf_command + west_command, text=True, env=env)

    print(result.stdout)

    if result.returncode != 0:
        print("Build failed:", result.stderr)
        sys.exit(1)
    print("Build completed successfully.")

def main():
    parser = argparse.ArgumentParser(description="Build a Nordic NRF Connect application with sysbuild.")
    parser.add_argument("--sdk-version", required=True, help="The version of the SDK/toolchain to install.")
    parser.add_argument("--project-path", required=True, help="Path to the NRF Connect project.")
    parser.add_argument("--build-dir", required=True, help="Destination directory for the build output.")
    parser.add_argument("--board", default="nrf5340dk_nrf5340_cpuapp", help="Target board for the build (default: nrf5340dk_nrf5340_cpuapp).")
    parser.add_argument("--config", help="Optional configuration file for the build.")
    parser.add_argument("--install-dir", required=True, help="Directory to install the SDK and toolchain.")
    parser.add_argument("--custom-board-dir", help="Optional directory for custom board definitions.")
    args = parser.parse_args()

    install_toolchain_and_sdk(args.sdk_version, args.install_dir)
    launch_build(args.sdk_version, args.project_path, args.build_dir, args.board, args.config, args.install_dir, args.custom_board_dir)

if __name__ == "__main__":
    main()
